#include <string.h>
#include "arg.h"
#include "log.h"
#include "dep.h"
#include "generator.h"

/* for strue/sfalse */
#include "db.h"

int notree = 0;
char *notree_reason = NULL;

/* Runs when a custom command line argument is found
 returns true if no furhter argument processing should be done */
int hook_custom_arg(const char *key, const char *value)
{
	if (strcmp(key, "without-c99tree") == 0) {
		notree = 2;
		notree_reason = "it was disabled from the command line (--without-c99tree)";
	}
	return 0;
}


/* Runs before anything else */
int hook_preinit()
{
	return 0;
}

/* Runs after initialization */
int hook_postinit()
{
	report("Configuring libporty.\n");
	logprintf(0, "Configuring libporty.\n");
	return 0;
}

/* Runs after all arguments are read and parsed */
int hook_postarg()
{
	return 0;
}

/* Runs when things should be detected for the host system */
int hook_detect_host()
{
	require("fstools/ar", 1, 1);
	require("fstools/ranlib", 1, 1);
	require("fstools/mkdir", 1, 1);
	return 0;
}

#include "hooks_net.c"

/* Runs when things should be detected for the target system */
int hook_detect_target()
{
	if (require("libs/parser/expat/ldflags", 1, 0)) {
		notree = 1;
		notree_reason = "of missing libexpat";
	}

	if (!notree && require("parsgen/flex", 1, 0)) {
		notree = 1;
		notree_reason = "of missing flex";
	}

	if (!notree && require("parsgen/bison", 1, 0)) {
		notree = 1;
		notree_reason = "of missing bison";
	}

	libporty_net_detect_target();

	/* detect math lib corner cases for math_wrap.c */
	require("libs/math/cc/log/*", 1, 0);

	if (notree)
		put("local/c99tree/enabled", sfalse);
	else
		put("local/c99tree/enabled", strue);

	return 0;
}

/* Runs after detection hooks, should generate the output (Makefiles, etc.) */
int hook_generate()
{
	printf("Generating Makefiles for src/...\n");
	generate("../src/Makefile.in", "../src/Makefile");
	printf("Generating Makefiles for c99tree...\n");
	generate("../src/porty/c99tree-cc/Makefile.config.in", "../src/porty/c99tree-cc/Makefile.config");
	printf("Generating config.h for libporty...\n");
	generate("../src/libporty/config.h.in", "../src/libporty/config.h");
	generate("../src/libporty/Makefile.config.in", "../src/libporty/Makefile.config");

	generate("../src/libporty/net/os_includes.h.in", "../src/libporty/net/os_includes.h");
	generate("../src/libporty/net_resolver/resolver_inc.h.in", "../src/libporty/net_resolver/resolver_inc.h");
	generate("../src/libporty/host/fs_includes.h.in", "../src/libporty/host/fs_includes.h");
	generate("../src/libporty/host/time_includes.h.in", "../src/libporty/host/time_includes.h");
	generate("../src/libporty/host/types.h.in", "../src/libporty/host/types.h");
	generate("../src/libporty/net_async/Makefile.in", "../src/libporty/net_async/Makefile");
	generate("../src/regression/libporty/Makefile.in", "../src/regression/libporty/Makefile");


	printf("\n\n=== Result ===\n");
	if (notree) {
		printf("libporty is configured.\n");
		printf("Can't configure (and build) c99tree because %s\n", notree_reason);
		if (notree == 1)
			printf("In case c99tree and porty is needed, please install the missing software and rerun configure\n");
	}
	else
		printf("libporty and c99tree and porty are configured.\n");
	return 0;
}

/* Runs before everything is uninitialized */
void hook_preuninit()
{
}

/* Runs at the very end, when everything is already uninitialized */
void hook_postuninit()
{
}

