/*
    c99tree - input layer, c parser wrapper
    Copyright (C) 2010-2015  Tibor Palinkas

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "c99tree-cc.h"
#include "tree_load.h"
#include "scope.h"

c99c_node_t *c99c_tree_load_c(c99tree_cc_t *ctx, const char *filename)
{
	c99c_node_t *gram_root, *tok_root;

	if (filename != NULL) {
		ctx->yin = fopen(filename, "r");
		if (ctx->yin == NULL) {
			fprintf(stderr, "c99tree error: can't open %s\n", filename);
			return NULL;
		}
		ctx->close_yyin = 1;
		c99c_lex_init(&ctx->scanner);
		c99c_set_in (ctx->yin , ctx->scanner);
	}
	else {
		c99c_lex_init(&ctx->scanner);
		filename = "<stdin>"; /* if no file name from the command line, use stdin */
	}

	ctx->AST = NULL;
	ctx->TOKT = NULL;
	ctx->close_yyin = 0;

	c99c_scope_enter(ctx);
	c99c_loc_file_open(ctx, NULL, NULL); /* file name: NULL means wait until the first preproc tag */

	gram_root = C99Nalloc(ctx, T_GRAMMAR_TREE, NULL, NULL);
	gram_root->loc.soffs = -1;
	gram_root->loc.eoffs = -1;
	gram_root->loc.twin_parent = NULL;
	gram_root->loc.twin_parent_uid = -1;

	tok_root = C99Nalloc(ctx, T_TOKEN_TREE, NULL, NULL);
	tok_root->loc.soffs = -1;
	tok_root->loc.eoffs = -1;
	tok_root->loc.twin_parent = NULL;
	tok_root->loc.twin_parent_uid = -1;

	c99c_parse(ctx);
	c99c_scope_leave(ctx);

	tok_root->children = ctx->TOKT;
	gram_root->children = ctx->AST;
	gram_root->next = tok_root;
	
	return gram_root;
}

void c99c_tree_load_cleanup_c(c99tree_cc_t *ctx)
{
	if (ctx->close_yyin)
		fclose(ctx->yin);
	c99c_lex_destroy(ctx->scanner);
}
